/*
 * com.gregor.rrd.FetchResults
 *
 * $Id: FetchResults.java,v 1.1.1.1 2004/08/17 01:07:04 djgregor Exp $
 *
 * Copyright (c) 2003 Daniel J. Gregor, Jr.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.gregor.rrd;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

import java.util.Date;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Iterator;

public class FetchResults extends TreeMap {
    private String[] m_dataSources = new String[0];
    private boolean m_gotHeaders = false;

    // DEC: Not in decompiled version
    public FetchResults() {}

    public void setDataSources(String[] dataSources) {
	m_dataSources = dataSources;
    }

    public String[] getDataSources() {
	return m_dataSources;
    }

    public Double[] get(Long timestamp) {
	return (Double[]) super.get(timestamp);
    }

    public double[] getColumn(String dataSource) {
	int column = -1;
	int i;
	Iterator it;

	for (i = 0; i < m_dataSources.length; i++) {
	    if (m_dataSources[i].equals(dataSource)) {
		column = i;
		break;
	    }
	}

	if (column == -1) {
	    return null;
	}

	double[] data = new double[size()];

	for (i = 0, it = keySet().iterator(); it.hasNext(); i++) {
	    data[i] = get((Long) it.next())[column].doubleValue();
	}

	return data;
    }

    public static double mean(double[] individuals) {
	double sum = 0.0;
	int count = 0;

	for (int n = 0; n < individuals.length; n++) {
	    if (!Double.isNaN(individuals[n]) &&
		    !Double.isInfinite(individuals[n])) {
		sum += individuals[n];
		count++;
	    }
	}

	return sum / (double) count;
    }

    public static double stddev(double[] individuals, double mean) {
	double sqdiff = 0.0;
	int count = 0;
	
	for (int n = 0; n < individuals.length; n++) {
	    if (!Double.isNaN(individuals[n]) &&
		    !Double.isInfinite(individuals[n])) {
		sqdiff += Math.pow(individuals[n] - mean, 2.0);
		count++;
	    }
	}

	return (count < 2) ? Double.NaN : Math.sqrt(sqdiff/(count - 1));

    }

    public static double stddev(double[] individuals) {
	return stddev(individuals, mean(individuals));
    }

    public static FetchResults parseResults(BufferedReader reader)
	    throws IOException, RRDException, NumberFormatException {
	FetchResults r = new FetchResults();
	String l;

	while ((l = reader.readLine()) != null) {
	    r.parse(l);
	}
	return r;
    }

    public void parse(String l) throws RRDException, NumberFormatException {
	StringTokenizer t = new StringTokenizer(l, " \t\n\r\014:");
	int i = t.countTokens();

	if (i == 0) {
	    return;
	}
		
	if (!m_gotHeaders) {
	    ArrayList a = new ArrayList();
	    a.ensureCapacity(i);

	    while (t.hasMoreTokens()) {
		a.add(t.nextToken());
	    }

	    setDataSources((String[]) a.toArray(new String[0]));

	    m_gotHeaders = true;
	} else {
	    if (i != (getDataSources().length + 1)) {
		throw new RRDException("Number of expected fields (" +
				      (getDataSources().length + 1) +
				      ") does not " +
				      "equal number of fields found " +
				      "in record (" + i + ")");
	    }
	    Double[] data = new Double[getDataSources().length];
	    long time = Long.parseLong(t.nextToken()) * 1000L;
	    for (int j = 0; t.hasMoreTokens(); j++) {
		String s = t.nextToken();
		data[j] = s.equalsIgnoreCase("NaN") ?
		    new Double(Double.NaN) :
		    new Double(s);
	    }
	    put(new Long(time), data);
	}
    }

    public String toString() {
	StringBuffer b = new StringBuffer();

	b.append("timestamp");
	for (int i = 0; i < m_dataSources.length; i++) {
	    b.append(" ");
	    b.append(m_dataSources[i]);
	}
	b.append("\n");

	for (Iterator i = keySet().iterator(); i.hasNext(); ) {
	    Long timestamp = (Long) i.next();
	    b.append(new Date(timestamp.longValue()).toString());

	    Double[] data = get(timestamp);
	    for (int j = 0; j < data.length; j++) {
		b.append(" ");
		b.append(data[j].toString());
	    }
	    b.append("\n");
	}

	return b.toString();
    }

    public static void main(String[] argv) {
	BufferedReader reader =
	    new BufferedReader(new InputStreamReader(System.in)); 
	FetchResults r = null;

	try {
	    r = parseResults(reader);
	} catch (IOException e) {
	    System.err.println("IO exception while reading data: " +
			       e.getMessage());
	    System.exit(1);
	} catch (RRDException e) {
	    System.err.println("RRD exception while reading data: " +
			       e.getMessage());
	    System.exit(1);
	}

	System.out.print(r.toString());
    }
}
